import {Component, ElementRef, HostListener, OnInit, ViewChild} from '@angular/core';
import {DataSourceService} from './services/data-source.service';
import {UserContextService} from './services/user-context.service';
import {RoleSelectorComponent} from './role-selector/role-selector.component';
import {MatIconRegistry} from '@angular/material/icon';
import {DomSanitizer} from '@angular/platform-browser';
import {DataCard, DataCardResponse} from '@nspop/gm-web-facade-api';
import {DialogService} from './modals/dialog/dialog.service';
import {PatientContextService} from './services/patient-context.service';
import {WindowService} from './services/window.service';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent implements OnInit{
  title = 'Graviditetsportalen';
  dataCard: DataCard | undefined;
  allDataCards: DataCardResponse[] = [];
  hasPatientInContext = false;
  hasEpisodeOfCareInContext = false;
  hasXBreakTheGlassReasonInContext = false;
  hasPrivatelyMarkedDataInContext = false;
  patientExists = false;
  feedbackModalOpen = false;

  @ViewChild('sidebar') sidebar?: ElementRef<HTMLElement>;

  constructor(
    public dataSourceService: DataSourceService,
    public userContextService: UserContextService,
    public dialogService: DialogService,
    private patientContextService: PatientContextService,
    private matIconRegistry: MatIconRegistry,
    private domSanitizer: DomSanitizer,
    private windowService: WindowService,
  ) {}

  public ngOnInit(): void {
    this.registerIcons();
    this.registerSmileys();
    this.userContextService.userContext$.pipe(
    );
    this.hasPatientInContext = !!this.patientContextService.getCurrentCpr();
    if (this.hasPatientInContext) {
      this.getDataCards();
    }

    this.patientContextService.cpr$.subscribe((cpr) => {
      this.hasPatientInContext = !!cpr;
      if (cpr) {
        if (this.dataCard) {
          this.dataSourceService.refresh();
        } else {
          this.getDataCards();
        }
      }
    });

    this.hasEpisodeOfCareInContext = !!this.patientContextService.getCurrentEpisodeOfCare();
    this.patientContextService.episodeOfCare$.subscribe((episodeOfCareId) => {
      this.hasEpisodeOfCareInContext = !!episodeOfCareId;
      this.dataCard = this.allDataCards.find(x => x.episodeOfCareIdentifier === episodeOfCareId);
    });

    this.hasXBreakTheGlassReasonInContext = !!this.patientContextService.getCurrentXBreakTheGlassReason();
    this.patientContextService.xBreakTheGlassReason$.subscribe((reason => {
      this.hasXBreakTheGlassReasonInContext = !!reason;
    }));

    this.hasPrivatelyMarkedDataInContext = !!this.patientContextService.getCurrentHasPrivatelyMarkedData();
    this.patientContextService.hasPrivatelyMarkedData$.subscribe((value => {
      this.hasPrivatelyMarkedDataInContext = !!value;
    }));

    // If patientExists is undefined, we don't know yet if the patient exists - proceed as if it did
    this.patientExists = this.patientContextService.getCurrentPatientExists() === undefined
      || !!this.patientContextService.getCurrentPatientExists();
    this.patientContextService.patientExists$.subscribe((exists) => {
      this.patientExists = exists === undefined || !!exists;
    });
  }

  private getDataCards(): void {
    this.dataSourceService.allDataCards$.subscribe((cards: DataCardResponse[]) => {
      this.allDataCards = cards;
      this.dataCard = cards?.[0];
      switch (cards?.[0]?.documentStatus) {
        case 'ACTIVE':
          this.patientContextService.setEpisodeOfCare(cards[0].episodeOfCareIdentifier);
          break;
        case 'NEW':
        case 'CLOSED':
        // TODO: add handlers here
      }
    });
  }

  private registerIcons(): void {
    this.matIconRegistry.addSvgIcon('ic-edit',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_edit.svg'));
    this.matIconRegistry.addSvgIcon('ic-view',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_view-faded.svg'));
    this.matIconRegistry.addSvgIcon('ic-edit-highlight',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_edit_highlighted.svg'));
    this.matIconRegistry.addSvgIcon('ic-trash',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_trash.svg'));
    this.matIconRegistry.addSvgIcon('ic-calendar',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_calendar.svg'));
    this.matIconRegistry.addSvgIcon('ic-more',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_more.svg'));
    this.matIconRegistry.addSvgIcon('ic-info',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_info.svg'));
    this.matIconRegistry.addSvgIcon('ic-add',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_add.svg'));
    this.matIconRegistry.addSvgIcon('ic-close',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_close-large.svg'));
    this.matIconRegistry.addSvgIcon('ic-care-plan',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_fagperson-mand.svg'));
    this.matIconRegistry.addSvgIcon('ic-overview',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_overblik.svg'));
    this.matIconRegistry.addSvgIcon('ic-add-colored',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_add-colored.svg'));
  }

  private registerSmileys(): void {
    const addSmiley = (name: string) => {
      this.matIconRegistry.addSvgIcon(`smiley-${name}`,
        this.domSanitizer.bypassSecurityTrustResourceUrl(`assets/img/smileys/smiley-${name}.svg`));
      this.matIconRegistry.addSvgIcon(`smiley-${name}@faded`,
        this.domSanitizer.bypassSecurityTrustResourceUrl(`assets/img/smileys/smiley-${name}@faded.svg`));
    };
    addSmiley('super-sad');
    addSmiley('sad');
    addSmiley('meh');
    addSmiley('happy');
    addSmiley('super-happy');
  }

  openChangeRoleDialog(): void {
   this.dialogService.show(RoleSelectorComponent, {
      width: 'fit-content',
    });

  }

  @HostListener('window:scroll')
  public allowYScrollFrame(): void {
    const {x, y} = this.windowService.getXYScroll();
    const style = this.sidebar?.nativeElement.style;
    if (style) {
      style.left = `${-x}px`;
      style.top = `${0}px`;
      if (y <= 72) {
        style.top = `${72 - y}px`;
      }
    }
  }

  openFeedbackDialog(): void {
    this.feedbackModalOpen = true;
  }

  public closePatient(): void {
    console.log('TODO: this should close the patient');
  }
}
