import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import {EditMeasurementsModalComponent} from './edit-measurements-modal.component';
import {DataSourceService} from '../../services/data-source.service';
import {of} from 'rxjs';
import {FormsModule, NgForm} from '@angular/forms';
import {AppModule} from '../../app.module';
import {map} from 'rxjs/operators';
import {By} from '@angular/platform-browser';

describe('EditMeasurementsModalComponent', () => {
  let component: EditMeasurementsModalComponent;
  let fixture: ComponentFixture<EditMeasurementsModalComponent>;
  let dataSourceService: DataSourceService;

  beforeEach(async () => {
    dataSourceService = {
      activeDataCard$: of({
        expectedDateOfDeliverySection: {},
        prenatalRiskAssessmentSection: {
          finalRiskAssessment: {
            multiplicityRiskAssessment: {}
          },
        },
        overallAssessmentSection: {prenatalCareAssessment: {numberOfFetuses: 2}}
      }),
      activities$: of([]),
      updateDataCard: () => of(),
      createUltrasoundObservations: () => of(),
      createRegularObservations: () => of()
    } as any;
    await TestBed.configureTestingModule({
      declarations: [ EditMeasurementsModalComponent ],
      imports: [FormsModule, AppModule],
      providers: [
        {provide: DataSourceService, useValue: dataSourceService},
        {provide: NgForm, useValue: new NgForm([], [])},
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(EditMeasurementsModalComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('fetches datacard and resets observationData on init', fakeAsync(() => {
    component.isOpen = true;
    component.ngOnChanges({isOpen: {}} as any);
    fixture.detectChanges();
    let accessed = false;
    dataSourceService.activeDataCard$ = of({overallAssessmentSection: {prenatalCareAssessment: {numberOfFetuses: 3}}} as any).pipe(
      map(x => {accessed = true; return x; })
    );
    component.ngOnInit();
    tick();
    expect(accessed).toBeTrue();
    expect(component.observationData?.fetuses?.length).toBe(3);
  }));

  it('allows save if activity was picked', fakeAsync(() => {
    component.isOpen = true;
    component.ngOnChanges({isOpen: {}} as any);
    component.observationType = 'regular';
    const spy = spyOn(dataSourceService, 'createRegularObservations').and.callThrough();
    component.observationData = {activityId: 'user_selected_some_id_in_form'} as any;
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(spy).toHaveBeenCalled();
  }));

  it('does not allow save if no activity was picked', fakeAsync(() => {
    component.isOpen = true;
    component.ngOnChanges({isOpen: {}} as any);
    component.observationType = 'regular';
    const spy = spyOn(dataSourceService, 'createRegularObservations').and.callThrough();
    component.observationData = {} as any;
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(spy).not.toHaveBeenCalled();
    const by = (str: string) => By.css(str);
    const activitySelector = fixture.debugElement.query(By.css('#activitySelector')).nativeElement;
    expect(activitySelector.classList.contains('ng-invalid')).toBeTrue();
  }));

  it('just creates observation when in "regular" mode', fakeAsync(() => {
    component.isOpen = true;
    component.ngOnChanges({isOpen: {}} as any);
    component.observationType = 'regular';
    const dataCardSpy = spyOn(dataSourceService, 'updateDataCard').and.callThrough();
    const regularObservationSpy = spyOn(dataSourceService, 'createRegularObservations').and.callThrough();
    component.observationData = {activityId: 'user_selected_some_id_in_form'} as any;
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(dataCardSpy).not.toHaveBeenCalled();
    expect(regularObservationSpy).toHaveBeenCalled();
  }));

  it('saves datacard and creates observations when in "ultrasound" mode', fakeAsync(() => {
    component.isOpen = true;
    component.ngOnChanges({isOpen: {}} as any);
    component.observationType = 'ultrasound';
    const dataCardSpy = spyOn(dataSourceService, 'updateDataCard').and.callThrough();
    const scanObservationSpy = spyOn(dataSourceService, 'createUltrasoundObservations').and.callThrough();
    component.observationData = {activityId: 'user_selected_some_id_in_form'} as any;
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(dataCardSpy).toHaveBeenCalled();
    expect(scanObservationSpy).toHaveBeenCalled();
  }));

  it('handles that active datacard could be undefined', fakeAsync(() => {
    component = new EditMeasurementsModalComponent({activeDataCard$: of()} as any);
    component.ngOnInit();
    tick();
    expect(component.dataCard).toBeUndefined();
  }));
});
