import {fakeAsync, flush, TestBed} from '@angular/core/testing';

import { DataSourceService } from './data-source.service';

import {DataCardResponse} from '@nspop/gm-web-facade-api';
import {of, Subject} from 'rxjs';
import {HttpClientTestingModule} from '@angular/common/http/testing';
import {PatientContextService} from './patient-context.service';
import {Router} from '@angular/router';
import {appUrls} from '../../utils/urls';
import {DataCardService} from '../api/data-card.service';
import {CarePlanService} from '../api/care-plan.service';
import {ObservationService} from '../api/observation.service';

describe('DataSourceService', () => {
  let service: DataSourceService;
  let dataCardServiceStub: DataCardService;
  let episodeOfCareStub = new Subject<string | undefined>();
  let patientContextServiceStub: PatientContextService;
  let documentStatusStub: string;
  let routerStub: Router;

  beforeEach(async () => {
    episodeOfCareStub = new Subject<string | undefined>();

    dataCardServiceStub = {
      maternityDataCardPut: (id: string, version: string) => of({documentUniqueId: id, version: String(Number(version) + 1)} as any),
      maternityDataCardGetAll: () => of([
        {documentUniqueId: '1234', version: '5', episodeOfCareIdentifier: '1', documentStatus: documentStatusStub} as any,
        {documentUniqueId: '789', version: '3', episodeOfCareIdentifier: '2', documentStatus: documentStatusStub} as any,
      ]),
    } as any;

    patientContextServiceStub = {
      getCurrentEpisodeOfCare: () => '1',
      getCurrentXBreakTheGlassReason: () => undefined,
      episodeOfCare$: episodeOfCareStub,
      setHasPrivatelyMarkedData: () => {},
      getCurrentHasPrivatelyMarkedData: () => false,
    } as any;

    routerStub = {
      navigate: () => {}
    } as any;

    await TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub},
        {provide: CarePlanService, useValue: {}},
        {provide: ObservationService, useValue: {}},
        {provide: PatientContextService, useValue: patientContextServiceStub},
        {provide: Router, useValue: routerStub},
      ]
    }).compileComponents();
    service = TestBed.inject(DataSourceService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('updates activeDataCard$ observable after saving datacard', fakeAsync(() => {
    const invocations: Array<DataCardResponse|undefined> = [];
    service.activeDataCard$.subscribe((dataCard) => {
      invocations.push(dataCard);
    });
    service.updateDataCard({} as any, '123', '1').toPromise();
    flush();
    expect(invocations.length).toBeGreaterThanOrEqual(1);
    const card = invocations[invocations.length - 1];
    expect(card).toBeDefined();
    expect(card?.documentUniqueId).toBe('123');
    expect(card?.version).toBe('2');
  }));

  it('updates activeDataCard$ when asked to refresh', fakeAsync(() => {
    let called = false;
    service.activeDataCard$.subscribe((dataCard) => {
      expect(dataCard).toBeDefined();
      expect(dataCard?.documentUniqueId).toBe('1234');
      expect(dataCard?.version).toBe('5');
      called = true;
    });
    service.refresh();
    flush();
    expect(called).toBeTrue();
  }));

  it('updates activeDataCard$ when episodeOfCare$ changes', fakeAsync(() => {
    const invocations: Array<DataCardResponse|undefined> = [];
    service.activeDataCard$.subscribe((dataCard) => {
        invocations.push(dataCard);
    });
    episodeOfCareStub.next('2');
    flush();
    expect(invocations.length).toBeGreaterThanOrEqual(1);
    const card = invocations[invocations.length - 1];
    expect(card).toBeDefined();
    expect(card?.documentUniqueId).toBe('789');
    expect(card?.version).toBe('3');
  }));

  describe('when getting all dataCards', () => {

    describe('and documentStatus === PARTIAL', () => {
      beforeEach(() => {
        documentStatusStub = 'PARTIAL';
      });

      it('sets patientContext.hasPrivatelyMarkedData to true', () => {
        spyOn(patientContextServiceStub, 'setHasPrivatelyMarkedData');
        service.allDataCards$.subscribe();
        expect(patientContextServiceStub.setHasPrivatelyMarkedData).toHaveBeenCalledTimes(1);
        expect(patientContextServiceStub.setHasPrivatelyMarkedData).toHaveBeenCalledWith(true);
      });

      it('navigates to appUrls.privatelyMarked', () => {
        patientContextServiceStub.getCurrentHasPrivatelyMarkedData = () => true;
        spyOn(routerStub, 'navigate');
        service.allDataCards$.subscribe();
        expect(routerStub.navigate).toHaveBeenCalledTimes(1);
        expect(routerStub.navigate).toHaveBeenCalledWith([appUrls.privatelyMarked]);
      });

      it('does not navigate to appUrls.privatelyMarked when xBreakTheGlassReason is set', () => {
        patientContextServiceStub.getCurrentHasPrivatelyMarkedData = () => true;
        patientContextServiceStub.getCurrentXBreakTheGlassReason = () => 'Værdispring';
        spyOn(routerStub, 'navigate');
        service.allDataCards$.subscribe();
        expect(routerStub.navigate).toHaveBeenCalledTimes(0);
      });
    });

    describe('and documentStatus != PARTIAL ', () => {
      beforeEach(() => {
        documentStatusStub = 'WHATEVER';
      });

      it('sets patientContext.hasPrivatelyMarkedData to false', () => {
        spyOn(patientContextServiceStub, 'setHasPrivatelyMarkedData');
        service.allDataCards$.subscribe();
        expect(patientContextServiceStub.setHasPrivatelyMarkedData).toHaveBeenCalledTimes(1);
        expect(patientContextServiceStub.setHasPrivatelyMarkedData).toHaveBeenCalledWith(false);
      });

      it('does not navigate to appUrls.privatelyMarked', () => {
        patientContextServiceStub.getCurrentHasPrivatelyMarkedData = () => false;
        spyOn(routerStub, 'navigate');
        service.allDataCards$.subscribe();
        expect(routerStub.navigate).toHaveBeenCalledTimes(0);
      });
    });
  });
});
