import {Injectable} from '@angular/core';
import {BehaviorSubject, merge, Observable, of, Subject} from 'rxjs';
import {
  CarePlan,
  Activity,
  DataCard,
  DataCardResponse,
  CreateRegularObservationsRequest,
  CreateUltrasoundObservationsRequest,
} from '@nspop/gm-web-facade-api';
import {CarePlanService} from '../api/care-plan.service';
import {DataCardService} from '../api/data-card.service';
import {map, shareReplay, switchMap, tap} from 'rxjs/operators';
import {UserContextService} from './user-context.service';
import {PatientContextService} from './patient-context.service';
import {Router} from '@angular/router';
import {appUrls} from '../../utils/urls';
import {ObservationService} from '../api/observation.service';

/**
 * Shared data service
 */
@Injectable({
  providedIn: 'root'
})
export class DataSourceService {
  constructor(
    private observationsService: ObservationService,
    private carePlanService: CarePlanService,
    private dataCardService: DataCardService,
    private userContextService: UserContextService,
    private patientContextService: PatientContextService,
    private router: Router
  ) {
  }

  // activate data Loading
  private innerDataLoadingActivator: Subject<undefined> = new BehaviorSubject(undefined);

  private innerActiveDataCard: Subject<DataCardResponse | undefined> = new Subject<DataCardResponse | undefined>();
  private innerActiveCarePlan: Subject<CarePlan | undefined> = new Subject<CarePlan | undefined>();

  // load data when innerDataLoadingActivator is emitting and when role is selected
  private dataLoadingActivator: Observable<undefined> = merge(
      this.innerDataLoadingActivator,
      this.userContextService.contextIsInitialized$
  ).pipe(map(_ => undefined));

  public activeDataCard$: Observable<DataCardResponse | undefined> = merge(
    this.patientContextService.episodeOfCare$.pipe(
      switchMap((id: string | undefined) => this.getActiveDataCard(id)),
      shareReplay()
    ),
    this.dataLoadingActivator.pipe(
        switchMap(() => this.dataCardService.getAll().pipe(
          map((cards: DataCardResponse[]) =>
            cards.find((x) => x.episodeOfCareIdentifier === this.patientContextService.getCurrentEpisodeOfCare()))
        )),
        shareReplay(),
    ),
    this.innerActiveDataCard
  );

  public allDataCards$: Observable<DataCardResponse[]> = this.dataLoadingActivator.pipe(
    switchMap(() => this.dataCardService.getAll()
      .pipe(
        tap((x?: DataCardResponse[]) => this.innerActiveDataCard.next(
          x?.find(card => card.episodeOfCareIdentifier === this.patientContextService.getCurrentEpisodeOfCare()))
        ),
        map((x?: DataCardResponse[]) => {
          if (x) {
            const {documentUniqueId, version, episodeOfCareIdentifier, ...dataCard} = x[0];
            // @ts-ignore // TODO: Remove when dataCard.documentStatus model supports 'PARTIAL' value
            this.patientContextService.setHasPrivatelyMarkedData(dataCard.documentStatus === 'PARTIAL');

            const needConsent = this.patientContextService.getCurrentHasPrivatelyMarkedData()
              && !this.patientContextService.getCurrentXBreakTheGlassReason();
            if (needConsent) {
              this.router.navigate([appUrls.privatelyMarked]);
            }
          }
          return x ?? [];
        })
      )
    ),
    shareReplay()
  );

  public carePlan$: Observable<CarePlan | undefined> = this.dataLoadingActivator.pipe(
    switchMap(() => {
      const episodeOfCareId = this.patientContextService.getCurrentEpisodeOfCare();
      if (episodeOfCareId) {
        return this.carePlanService.getAll();
      }
      return of(undefined);
    }),
    shareReplay()
  );

  public activities$: Observable<Array<Activity>> = this.carePlan$.pipe(
      map((carePlan: CarePlan | undefined) => carePlan?.activities || [])
    );

  public getActivityById(id: string): Observable<Activity|null> {
    return this.carePlan$.pipe(
      map((careplan?: CarePlan) => careplan?.activities?.find(activity => activity.id === id) || null)
    );
  }

  public updateDataCard(
    resource: DataCard,
    documentUniqueId: string,
    version: string,
  ): Observable<DataCardResponse> {
    return this.dataCardService.update(resource, documentUniqueId, `${version}`).pipe(
      tap((x: DataCardResponse) => {
        this.innerActiveDataCard.next(x);
      })
    );
  }

  public updateCarePlan(
    resource: CarePlan,
    id: string,
    version: string,
  ): Observable<CarePlan> {
    return this.carePlanService.update(resource, id, version).pipe(
      tap((x: CarePlan) => {
        this.innerActiveCarePlan.next(x);
      })
    );
  }

  public createUltrasoundObservations(observations: CreateUltrasoundObservationsRequest): Observable<void> {
    return this.observationsService.createUltrasoundObservations(observations);
  }

  public createRegularObservations(observations: CreateRegularObservationsRequest): Observable<void> {
    return this.observationsService.createRegularObservations(observations);
  }

  public refresh(): void {
    this.innerDataLoadingActivator.next();
  }

  private getActiveDataCard(episodeOfCareId?: string): Observable<DataCardResponse | undefined> {
    if (episodeOfCareId) {
      return this.allDataCards$.pipe(
        map((cards: DataCardResponse[]) => cards.find(x => x.episodeOfCareIdentifier === episodeOfCareId))
      );
    }
    return of(undefined);
  }
}
