import {ErrorHandler, Injectable, NgZone} from '@angular/core';
import {Router} from '@angular/router';
import {PatientContextService} from './patient-context.service';
import {appUrls} from '../../utils/urls';
import {HttpErrorResponse} from '@angular/common/http';
import {Observable, throwError} from 'rxjs';
import {deleteCookie} from '../../utils/cookies';
import {environment} from '../../environments/environment';

@Injectable({
  providedIn: 'root'
})
export class ErrorService implements ErrorHandler {

  constructor(private zone: NgZone,
              private router: Router,
              private patientContextService: PatientContextService) {
  }

  handleError(error: any): void {
    if (error instanceof CprDoesNotExistError) {
      this.patientContextService.setPatientExists(false);
    } else if (error instanceof AppError) {
      this.zone.run(() => this.router.navigate([appUrls.somethingWentWrong]));
    }
    console.error(error);
  }
  /**
   * Specific app error handler
   * @param error HttpErrorResponse
   */
  // handleHttpError(error: HttpErrorResponse): Observable<never> {
  //   // FIXME: This should be a real service so we can inject dialogService to tell users something is wrong
  //   this.redirectIfRequested(error);
  //   if (error.status === 404) {
  //     return throwError(new PageNotFoundError(error));
  //   }
  //   if (error.status === 400
  //     && !!error.error?.parameterViolations
  //     && !!error.error?.parameterViolations[0]?.message?.includes('must match')) {
  //     return throwError(new CprDoesNotExistError(error));
  //   }
  //   return throwError(new AppError(error));
  // }
  /**
   * Browser intercepts redirects in xhr calls and returns 200 OK
   * this will check if any login redirects were requested by SAML
   * @param error the httpErrorResponse
   */
  redirectIfRequested(error: HttpErrorResponse): void {
    const redirectHtmlStart = '<html><head><meta http-equiv="refresh" content=';
    // Regex for oiosaml.fragment Cookie: url=(.*?)\".*document\.cookie = '(.*)';

    if (error.status === 200 && error.error.text?.includes(redirectHtmlStart)) {
      console.log('should be redirected');
      const redirectUrlRegex = /url=(.*?)\"/;
      const matches = redirectUrlRegex.exec(error.error.text);
      if (matches !== null && matches.length > 1) {
        deleteCookie('JSESSIONID');
        window.location.assign(environment.base_path);
      }
    }
  }
}

// Custom App Errors
/**
 * The Generic Application error. Parent to all other errors.
 */
export interface GenericAppError {
  errorMessage: string;
  innerError?: Error;
}

/**
 * Top level error
 */
export class AppError implements GenericAppError {
  errorMessage = 'Der skete en ukendt fejl';
  constructor(public innerError?: Error) {
  }
}

/**
 * Expected error for 404
 */
export class PageNotFoundError extends AppError {
  errorMessage = 'Siden kunne ikke findes';
}

/**
 * Expected error for when patient does not exist
 */
export class CprDoesNotExistError extends AppError {
  errorMessage = 'Cpr findes ikke';
}

/**
 * Expected error for when patient does not exist
 */
export class BadInputError extends AppError {
  errorMessage = 'Input har ikke rigtig form';
}


